(function(){

  /* number of wins, keeps track of score*/
  var wins = 0;
  var curr_highscore = 0;

  /* used to reset the timer */
  var timer;

  /* Get container elements */
  var container = $('#container');
  var charscontainer = $('#chars');
  var infocontainer = $('#info');

  /* Get buttons */
  var startbutton = $('#intro button');
  var infobutton = $('#infos');
  var winbutton = $('#win button');
  var reloadbutton = $('#reload');
  var soundbutton = $('#sound');
  var errorbutton = $('#error button');
  var quitbutton = $('#quitbutton');
  var continuebutton = $('#continue');
  var reallyquitbutton = $('#reallyquit');

  /* Get sounds */
  var winsound = $('#winsound');
  var errorsound = $('#errorsound');

  /* Prepare canvas */
  var c = $('canvas')[0];
  //var c = document.querySelector('canvas');
  var cx = c.getContext('2d');
  var letter = null;
  var fontsize = 300;
  var paintcolour = [240, 240, 240];
  var textcolour = [255, 30, 20];
  var xoffset = 0;
  var yoffset = 0;
  var linewidth = 20;
  var pixels = 0;
  var letterpixels = 0;

  /* Mouse and touch events */
  var mousedown = false;
  var touched = false;
  var oldx = 0;
  var oldy = 0;

  /* Overall game presets */
  var state = 'intro';
  var sound = true;
  var currentstate;

  /* timer for the motivational statements*/
  function setTimers() {
    timer = window.setInterval(function() {
      var statements = ['You can do it!', 'keep going!', 'You\'re doing great!'];
      var i = Math.floor((Math.random()*statements.length-1));

      $('#statements').text(statements[i]);

      clearAndRestart();

    }, 5000);
  }

  function clearAndRestart() {
      window.setTimeout(function() {
        $('#statements').empty();
        clearTimeout(timer);
        setTimers();
      }, 2000);

  }

  /* get and set the highscore label */
  function getHighscore() {
    $.getJSON('/highscore', function(data) {
      $('#highscore').text('highscore: ' + data.highscore);
      curr_highscore = data.highscore;
    
      if (curr_highscore < wins) {
        reportHighscore();
        $('#highscore').text('highscore: ' + wins);
      }
    });
  }

  function reportHighscore() {
    $.post('/reportscore', {highscore: wins});
  }

  function init() {
    xoffset = container.offsetLeft;
    yoffset = container.offsetTop;
    fontsize = container.offsetHeight / 1.5;
    linewidth = container.offsetHeight / 19;
    paintletter();
    setstate('intro');

    setTimers();
    getHighscore();
  }

  function togglesound() {
    if (sound) {
      sound = false;
      soundbutton.className = 'navbuttonoff';
    } else {
      sound = true;
      soundbutton.className = 'navbutton';
    }
  }

  function showerror() {
    setstate('error');
    if (sound) {
      errorsound.play();
    }
    if (navigator.vibrate) {
      navigator.vibrate(100);
    }
  }

  function showinfo() {
    if (state !== 'info') {
      setstate('info');
    } else {
      setstate('play');
    }
  }

  function quit() {
    setstate('quit');

    if (wins == curr_highscore) {
      $('#quittext').text('Congratulations! You got a highscore of ' + wins);
    }
    else {
      $('#quittext').text('You got a score of ' + wins + '. Good job!');
    }
  }

  function continue_playing() {
    setstate('play');
  }

  function reallyquit() {
    window.location = "/";
  }

  function setstate(newstate) {
    state = newstate;
    container.className = newstate;
    currentsate = state;

  }
  function moreneeded() {
    setstate('play');
    mousedown = false;
  }
  function retry(ev) {
    mousedown = false;
    oldx = 0;
    oldy = 0;
    paintletter(letter);
  }
  function winner() {
    paintletter();
    
    wins++;
    $('#counter').text('score: ' + wins);

    clearTimeout(timer);
    setTimers();

    getHighscore();
  }
  function start() {
    paintletter(letter);
  }
  function cancel() {
    paintletter();
  }
  function paintletter(retryletter) {
    var chars = charscontainer.html().split('');
    letter = retryletter ||
             chars[parseInt(Math.random() * chars.length,10)];
    c.width = container.offsetWidth;
    c.height = container.offsetHeight;
    cx.font = 'bold ' + fontsize + 'px Open Sans';
    cx.fillStyle = 'rgb(' + textcolour.join(',') + ')';
    cx.strokeStyle = 'rgb(' + paintcolour.join(',') + ')';
    cx.shadowOffsetX = 2;
    cx.shadowOffsetY = 2;
    cx.shadowBlur = 4;
    cx.shadowColor = '#666';

    cx.textBaseline = 'baseline';
    cx.lineWidth = linewidth;
    cx.lineCap = 'round';
    cx.fillText(
      letter,
      (c.width - cx.measureText(letter).width) / 2,
      (c.height / 1.3)
    );

    pixels = cx.getImageData(0, 0, c.width, c.height);
    letterpixels = getpixelamount(
      textcolour[0],
      textcolour[1],
      textcolour[2]
    );
    cx.shadowOffsetX = 0;
    cx.shadowOffsetY = 0;
    cx.shadowBlur = 0;
    cx.shadowColor = '#333';
    setstate('play');
  }

  function getpixelamount(r, g, b) {
    var pixels = cx.getImageData(0, 0, c.width, c.height);
    var all = pixels.data.length;
    var amount = 0;
    for (i = 0; i < all; i += 4) {
      if (pixels.data[i] === r &&
          pixels.data[i+1] === g &&
          pixels.data[i+2] === b) {
        amount++;
      }
    }
    return amount;
  }

  function paint(x, y) {
    var rx = x - xoffset;
    var ry = y - yoffset;
    var colour = pixelcolour(x, y);
    if( colour.r === 0 && colour.g === 0 && colour.b === 0) {
      showerror();
    } else {
      cx.beginPath();
      if (oldx > 0 && oldy > 0) {
        cx.moveTo(oldx, oldy);
      }
      cx.lineTo(rx, ry);
      cx.stroke();
      cx.closePath();
      oldx = rx;
      oldy = ry;
    }
  }

  function pixelcolour(x, y) {
    var index = ((y * (pixels.width * 4)) + (x * 4));
    return {
      r:pixels.data[index],
      g:pixels.data[index + 1],
      b:pixels.data[index + 2],
      a:pixels.data[index + 3]
    };
  }

  function pixelthreshold() {
    if (state !== 'error') {
      if (getpixelamount(
        paintcolour[0],
        paintcolour[1],
        paintcolour[2]
      ) / letterpixels > 0.35) {
       setstate('win');
       if (sound) {
         winsound.play();
       }
      }
    }
  }

  /* Mouse event listeners */

  function onmouseup(ev) {
    ev.preventDefault();
    oldx = 0;
    oldy = 0;
    mousedown = false;
    pixelthreshold();
  }
  function onmousedown(ev) {
    ev.preventDefault();
    mousedown = true;
  }
  function onmousemove(ev) {
    ev.preventDefault();
    if (mousedown) {
      paint(ev.clientX, ev.clientY);
      ev.preventDefault();
    }
  }

  /* Touch event listeners */

  function ontouchstart(ev) {
    touched = true;
  }
  function ontouchend(ev) {
    touched = false;
    oldx = 0;
    oldy = 0;
    pixelthreshold();
  }
  function ontouchmove(ev) {
    if (touched) {
      paint(
        ev.changedTouches[0].pageX,
        ev.changedTouches[0].pageY
      );
      ev.preventDefault();
    }
  }

  /* Button event handlers */

  errorbutton.on('click', retry, false);
  infobutton.on('click', showinfo, false);
  reloadbutton.on('click', cancel, false);
  soundbutton.on('click', togglesound, false);
  winbutton.on('click', winner, false);
  startbutton.on('click', start, false);
  quitbutton.on('click', quit, false);
  infocontainer.on('click', showinfo, false);
  continuebutton.on('click', continue_playing, false);
  reallyquitbutton.on('click', reallyquit, false);

  /* Canvas event handlers */

  c.addEventListener('mouseup', onmouseup, false);
  c.addEventListener('mousedown', onmousedown, false);
  c.addEventListener('mousemove', onmousemove, false);
  c.addEventListener('touchstart', ontouchstart, false);
  c.addEventListener('touchend', ontouchend, false);
  c.addEventListener('touchmove', ontouchmove, false);

  window.addEventListener('load',init, false);
  window.addEventListener('resize',init, false);

})();
